import unittest

from djcopybook.fixedwidth import fields
from djcopybook.fixedwidth.tests import record_helper


class FragmentWithNewlineFieldTests(unittest.TestCase):

    def test_is_subclass_of_fragment_field(self):
        self.assertTrue(issubclass(fields.FragmentWithNewlineField, fields.FragmentField))

    def test_to_python_returns_record_instance_when_given_string(self):
        f = fields.FragmentWithNewlineField(record=record_helper.RecordOne)
        python_val = f.to_python('AAAAA1111111')
        self.assertIsInstance(python_val, record_helper.RecordOne)

    def test_to_record_returns_record_class_defaults_when_value_is_None(self):
        f = fields.FragmentWithNewlineField(record=record_helper.RecordOne)
        self.assertEqual("AA   0000000\n", f.to_record(None))

    def test_to_record_will_modify_the_length_by_one_to_account_for_newline(self):
        my_record = record_helper.RecordOne(field_one="aa", field_two=999)
        r = record_helper.RecordFour(frag=my_record, other_field="ZZZ")
        self.assertEqual(16, len(str(r)))

    def test_to_record_returns_string_of_record_when_has_explicit_values(self):
        my_record = record_helper.RecordOne(field_one="aa", field_two=999)
        r = record_helper.RecordFour(frag=my_record, other_field="ZZZ")
        self.assertEqual("aa   0000999\nZZZ", r.to_record())

    def test_to_record_returns_string_of_record_when_explicitly_set(self):
        r = record_helper.RecordFour(other_field="ZZZ")
        self.assertEqual("AA   0000000\nZZZ", r.to_record())

    def test_creates_record_object_properly_from_string(self):
        r = record_helper.RecordFour.from_record("aaaaa9999999ZZZ")
        self.assertEqual("aaaaa", r.frag.field_one)
        self.assertEqual(9999999, r.frag.field_two)
        self.assertEqual("ZZZ", r.other_field)
        self.assertIsInstance(r.frag, record_helper.RecordOne)

    def test_creates_record_object_properly_from_string_with_multiple_values(self):
        r = record_helper.RecordFour()
        r.frag.field_one = "aaa"
        r.frag.field_two = 999

        self.assertEqual("aaa  0000999\nEEE", r.to_record())



from djcopybook import fixedwidth
from djcopybook.fixedwidth import fields


class RecordOne(fixedwidth.Record):
    field_one = fields.StringField(length=5, default="AA")
    field_two = fields.IntegerField(length=7)


class RecordTwo(RecordOne):
    auto_truncate = True

    field_three = fields.DecimalField(length=9)
    field_four = fields.DateField(length=2)


class RecordThree(fixedwidth.Record):
    frag = fields.FragmentField(record=RecordOne)
    other_field = fields.StringField(length=3, default="BBB")


class RecordFour(fixedwidth.Record):
    frag = fields.FragmentWithNewlineField(record=RecordOne)
    other_field = fields.StringField(length=3, default="EEE")


class RecordFive(fixedwidth.Record):
    garf = fields.FragmentWithNewlineField(RecordFour)
    threeve = fields.ListField(record=RecordThree, length=2)



import unittest

from djcopybook.fixedwidth import fields
from djcopybook.fixedwidth.tests import record_helper


class FragmentFieldTests(unittest.TestCase):

    def test_to_record_returns_record_class_defaults_when_value_is_None(self):
        field = fields.FragmentField(record=record_helper.RecordOne)
        self.assertEqual("AA   0000000", field.to_record(None))

    def test_to_python_returns_record_instance_when_none_given(self):
        f = fields.FragmentField(record=record_helper.RecordOne)
        python_val = f.to_python(None)
        self.assertIsInstance(python_val, record_helper.RecordOne)

    def test_to_python_returns_record_instance_when_given_string(self):
        f = fields.FragmentField(record=record_helper.RecordOne)
        python_val = f.to_python('AAAAA1111111')
        self.assertIsInstance(python_val, record_helper.RecordOne)

    def test_to_python_returns_record_instance_when_given_instance(self):
        record = record_helper.RecordOne()
        f = fields.FragmentField(record=record_helper.RecordOne)
        python_val = f.to_python(record)
        self.assertIsInstance(python_val, record_helper.RecordOne)

    def test_to_python_raises_type_error_when_receives_bad_input(self):
        f = fields.FragmentField(record=record_helper.RecordOne)
        with self.assertRaises(TypeError) as ctx:
            f.to_python(222)
        self.assertEqual("Redefined field must be a string or RecordOne instance.", str(ctx.exception))

    def test_to_record_returns_string_of_record_when_explicitly_set(self):
        r = record_helper.RecordThree(other_field="ZZZ")
        self.assertEqual("AA   0000000ZZZ", r.to_record())

    def test_to_record_returns_string_of_record_when_has_explicit_values(self):
        my_record = record_helper.RecordOne(field_one="aa", field_two=999)
        r = record_helper.RecordThree(frag=my_record, other_field="ZZZ")
        self.assertEqual("aa   0000999ZZZ", r.to_record())

    def test_creates_record_object_properly_from_string(self):
        r = record_helper.RecordThree.from_record("aaaaa9999999ZZZ")
        self.assertEqual("aaaaa", r.frag.field_one)
        self.assertEqual(9999999, r.frag.field_two)
        self.assertEqual("ZZZ", r.other_field)
        self.assertIsInstance(r.frag, record_helper.RecordOne)

    def test_creates_record_object_properly_from_unicode(self):
        r = record_helper.RecordThree.from_record(u"aaaaa9999999ZZZ")
        self.assertEqual("aaaaa", r.frag.field_one)
        self.assertEqual(9999999, r.frag.field_two)
        self.assertEqual("ZZZ", r.other_field)
        self.assertIsInstance(r.frag, record_helper.RecordOne)

    def test_creates_record_object_properly_from_string_with_multiple_values(self):
        r = record_helper.RecordThree()
        r.frag.field_one = "aaa"
        r.frag.field_two = 999

        self.assertEqual("aaa  0000999BBB", r.to_record())

    def test_populates_nested_dictionaries(self):
        input_data = {'garf': {'frag': {'field_one': 'abc', 'field_two': 100}}}
        my_record = record_helper.RecordFive(**input_data)
        self.assertEqual(my_record.garf.frag.field_one, 'abc')
        self.assertEqual(my_record.garf.frag.field_two, 100)



import unittest

from djcopybook.fixedwidth import fields


class DecimalFieldTests(unittest.TestCase):

    def test_to_record_returns_string_value_padded_to_length(self):
        field = fields.DecimalField(length=6, decimals=1)
        self.assertEqual("0010.0", field.to_record(10))

    def test_to_record_returns_padded_zeros_when_value_is_None(self):
        field = fields.DecimalField(length=7, decimals=2)
        self.assertEqual("0000.00", field.to_record(None))

    def test_to_python_turns_value_to_float(self):
        field = fields.DecimalField(length=5)
        python_val = field.to_python("10.12")
        self.assertEqual(10.12, python_val)
        self.assertIsInstance(python_val, float)

    def test_to_python_returns_none_when_value_is_none(self):
        field = fields.DecimalField(length=5)
        self.assertEqual(None, field.to_python(None))

    def test_to_python_returns_none_when_value_is_empty_string(self):
        field = fields.DecimalField(length=5)
        self.assertEqual(None, field.to_python("     "))

    def test_to_python_raises_value_error_when_value_given_has_characters(self):
        field = fields.DecimalField(length=5)
        with self.assertRaises(ValueError):
            self.assertEqual(None, field.to_python("0001A"))



from datetime import date, datetime
import unittest

from djcopybook.fixedwidth import fields


class DateTimeFieldTests(unittest.TestCase):

    def setUp(self):
        self.sut = fields.DateTimeField(length=14, format="%Y%m%d%H%M%S")

    def test_to_record_returns_formatted_datetime(self):
        self.assertEqual("20120101010101", self.sut.to_record(datetime(2012, 1, 1, 1, 1, 1)))

    def test_to_record_returns_empty_string_of_correct_length_when_none(self):
        self.assertEqual("              ", self.sut.to_record(None))

    def test_to_record_returns_empty_string_of_correct_length_when_empty_string(self):
        self.assertEqual("              ", self.sut.to_record(""))

    def test_to_python_returns_datetime_object(self):
        self.assertEqual(datetime(2012, 1, 1, 1, 1, 1), self.sut.to_python("20120101010101"))

    def test_to_python_returns_datetime_object_when_passed_a_date_object(self):
        self.assertEqual(datetime(2012, 1, 1, 0, 0, 0), self.sut.to_python(date(2012, 1, 1)))

    def test_to_python_returns_datetime_object_when_passed_a_datetime_object(self):
        self.assertEqual(datetime(2012, 1, 1, 1, 1, 1), self.sut.to_python(datetime(2012, 1, 1, 1, 1, 1)))

    def test_to_python_returns_none_when_empty_string(self):
        self.assertEqual(None, self.sut.to_python(""))

    def test_to_python_returns_none_when_none(self):
        self.assertEqual(None, self.sut.to_python(None))

    def test_to_python_returns_datetime_object_when_passed_string(self):
        self.assertEqual(datetime(2012, 1, 1, 1, 1, 1), self.sut.to_python("20120101010101"))

    def test_to_python_returns_datetime_object_when_unicode(self):
        self.assertEqual(datetime(2012, 1, 1, 1, 1, 1), self.sut.to_python(u"20120101010101"))



import unittest

from djcopybook.fixedwidth import fields


class PostalCodeFieldTests(unittest.TestCase):

    def setUp(self):
        self.field = fields.PostalCodeField()

    def test_postal_code_field_length_is_set_to_nine(self):
        self.assertEqual(9, self.field.length)

    def test_to_record_returns_string_of_value_padded_to_length_if_string(self):
        self.assertEqual("AA       ", self.field.to_record("AA"))

    def test_to_record_returns_padded_blank_string_when_value_is_None(self):
        self.assertEqual("         ", self.field.to_record(None))

    def test_to_record_returns_number_of_value_padded_to_right_of_length_if_number(self):
        self.assertEqual("990000000", self.field.to_record(99))
        self.assertEqual("990000000", self.field.to_record("99"))

    def test_to_python_turns_value_to_string(self):
        python_val = self.field.to_python("K1A XXX")
        self.assertEqual("K1A XXX", python_val)
        self.assertIsInstance(python_val, str)

    def test_to_python_turns_value_to_string_if_integer(self):
        python_val = self.field.to_python(50401)
        self.assertEqual("50401", python_val)
        self.assertIsInstance(python_val, str)



import unittest

from djcopybook.fixedwidth import fields


class IntegerFieldTests(unittest.TestCase):

    def test_to_record_returns_string_of_value_padded_to_length(self):
        field = fields.IntegerField(length=5)
        self.assertEqual("00012", field.to_record(12))

    def test_to_record_returns_padded_zeros_when_value_is_None(self):
        field = fields.IntegerField(length=5)
        self.assertEqual("00000", field.to_record(None))

    def test_to_python_turns_value_to_int(self):
        field = fields.IntegerField(length=5)
        python_val = field.to_python("10")
        self.assertEqual(10, python_val)
        self.assertIsInstance(python_val, int)

    def test_to_python_returns_none_when_value_is_none(self):
        field = fields.IntegerField(length=5)
        self.assertEqual(None, field.to_python(None))

    def test_to_python_returns_none_when_value_is_empty_string(self):
        field = fields.IntegerField(length=5)
        self.assertEqual(None, field.to_python("     "))

    def test_to_python_raises_value_error_when_value_given_has_characters(self):
        field = fields.IntegerField(length=5)
        with self.assertRaises(ValueError):
            self.assertEqual(None, field.to_python("0001A"))



import unittest

from djcopybook.fixedwidth import fields


class PaddingTests(unittest.TestCase):

    def test_str_padding_pads_right_with_spaces(self):
        self.assertEqual("ALM   ", fields.str_padding(6, "ALM"))
        self.assertEqual("5     ", fields.str_padding(6, 5))

    def test_int_right_padding_pads_right_with_zeros(self):
        self.assertEqual("110000", fields.int_padding(6, 11, "<"))
        self.assertEqual("110000", fields.int_padding(6, "11", "<"))

    def test_int_padding_pads_left_with_zeros(self):
        self.assertEqual("000010", fields.int_padding(6, 10))
        self.assertEqual("000010", fields.int_padding(6, "10"))

    def test_float_padding_pads_left_and_decimals_with_zeros(self):
        self.assertEqual("000010", fields.float_padding(6, 10, 0))
        self.assertEqual("000010", fields.float_padding(6, "10", 0))

        self.assertEqual("00300.12", fields.float_padding(8, 300.1234, 2))
        self.assertEqual("010.12", fields.float_padding(6, "10.1234", 2))
        self.assertEqual("010.10", fields.float_padding(6, "10.1", 2))
        self.assertEqual("010.00", fields.float_padding(6, 10, 2))

        self.assertEqual("000010.123", fields.float_padding(10, 10.1234, 3))
        self.assertEqual("0005.123", fields.float_padding(8, "5.1234", 3))
